"""Include processor."""

import re
import errno
import os
import os.path as ospa
from io import StringIO

from colorama import Fore

from . import gitutils


IMAGE_DIRS = ("", "images", "graphics")
IGNORE_DIRS = (".git", ".venv", ".vscode", "__pycache__", "foliantcache")

HEADING_PATTERN = re.compile(
    r"^(?P<hashes>\#+)\s*(?P<title>[^\#]+)\s*$",
    flags=re.MULTILINE
)
IMAGE_PATTERN = re.compile(r"\!\[(?P<caption>.*)\]\((?P<path>.+)\)")
INCLUDE_PATTERN = re.compile(
    r"\{\{\s*(\<(?P<repo>[^\#^\>]+)(#(?P<revision>[^\>]+))?\>)?" +
    r"(?P<path>[^\#]+?)" +
    r"(\#(?P<from_heading>[^:]*?)(:(?P<to_heading>.+?))?)?" +
    r"\s*(\|\s*(?P<options>.+))?\s*\}\}"
)
IGNORED_INCLUDE_PATTERN = re.compile(
    r"\<\<\s*(\<(?P<repo>[^\#^\>]+)(#(?P<revision>[^\>]+))?\>)?" +
    r"(?P<path>[^\#]+?)" +
    r"(\#(?P<from_heading>[^:]*?)(:(?P<to_heading>.+?))?)?" +
    r"\s*(\|\s*(?P<options>.+))?\s*\}\}"
)

def convert_value(value):
    """Attempt to convert a string to integer, float, or boolean. If nothing
    matches, return the original string.
    """

    try:
        return int(value)
    except ValueError:
        try:
            return float(value)
        except ValueError:
            try:
                return bool(value)
            except ValueError:
                return value


def extract_options(options_line):
    """Extract include options as a dictionary from a raw string.

    Options without values are treated as having boolean ``True`` value.
    """

    options = {}

    if not options_line:
        return options

    for option in (option.strip() for option in options_line.split(',')):
        option_parts = option.split(':')
        if len(option_parts) == 1:
            options[option_parts[0]] = True
        elif len(option_parts) == 2:
            options[option_parts[0]] = convert_value(option_parts[1])

    return options


def shift_headings(content, shift):
    """Shift all Markdown headings in a string by a given value. The shift can
    be positive or negative.
    """

    def sub(heading):
        new_heading_level = len(heading.group("hashes")) + shift
        return "%s %s" % ('#' * new_heading_level, heading.group("title"))

    return HEADING_PATTERN.sub(sub, content)


def find_top_heading_level(content):
    """Find the highest level heading (i.e. having the least '#'s)
    in a Markdown string."""

    result = float("inf")

    for heading in HEADING_PATTERN.findall(content):
        heading_level = heading.count("#")

        if heading_level < result:
            result = heading_level

    return result if result < float("inf") else 0


def adjust_headings(content, from_heading, to_heading=None,
                    options={}):
    """Cut part of Markdown string between two headings, set heading level,
    and remove top heading.

    If only the starting heading is defined, cut to the next heading
    of the same level. If no headings are defined, return the full string.

    Heading shift and top heading elimination are optional.
    """

    if from_heading:
        from_heading_pattern = re.compile(
            r"^\#+\s*%s\s*$" % from_heading,
            flags=re.MULTILINE
        )

        if not from_heading_pattern.findall(content):
            return ""

        from_heading_line = from_heading_pattern.findall(content)[0]
        from_heading_level = from_heading_line.count('#')

        result = from_heading_pattern.split(content)[1]

        if to_heading:
            to_heading_pattern = re.compile(
                r"^\#+\s*%s\s*$" % to_heading,
                flags=re.MULTILINE
            )

        else:
            to_heading_pattern = re.compile(
                r"^\#{1,%d}[^\#]+?$" % from_heading_level,
                flags=re.MULTILINE
            )

        result = to_heading_pattern.split(result)[0]

        if not options.get("nohead"):
            result = from_heading_line + result

        if options.get("sethead"):
            if options["sethead"] > 0:
                result = shift_headings(
                    result,
                    options["sethead"] - from_heading_level
                )

        return result

    else:
        content_buffer = StringIO(content)

        first_line = content_buffer.readline()

        if HEADING_PATTERN.fullmatch(first_line):
            from_heading_line = first_line
            from_heading_level = from_heading_line.count('#')
            result = content_buffer.read()

        else:
            from_heading_line = ''
            from_heading_level = find_top_heading_level(content)
            result = content

        if to_heading:
            to_heading_pattern = re.compile(
                r"^\#+\s*%s\s*$" % to_heading,
                flags=re.MULTILINE
            )
            result = to_heading_pattern.split(result)[0]

        if not options.get("nohead"):
            result = from_heading_line + result

        if options.get("sethead"):
            if options["sethead"] > 0:
                result = shift_headings(
                    result,
                    options["sethead"] - from_heading_level
                )

        return result


def find_image(image_path, start_dir, target_dir):
    """Locate an image in the disk based on the path from the image directive
    and the path to the directory with the document containing the image.

    The function looks for the image first in the document directory and then
    in all upper-level directories until it finds the image or reaches root.
    In each location, it checks the directory itself and the subdirectories
    ``images`` and ``graphics`` (defined in ``IMAGE_DIRS``).

    The result is adjusted relative to the target directory where the final
    document will be built.
    """

    def normabspath(path):
        return ospa.normcase(ospa.abspath(path))

    def is_root(path):
        return '' in ospa.split(normabspath(path))

    def normalize(path):
        return '/'.join(path.split(ospa.sep))

    def adjust(path):
        return ospa.relpath(path, target_dir)

    level = 0
    lookup_dir = ospa.join(start_dir, "../" * level)

    while not is_root(lookup_dir):
        for image_dir in (
            ospa.join(lookup_dir, image_dir) for image_dir in IMAGE_DIRS
        ):
            if ospa.isfile(ospa.join(image_dir, image_path)):
                return normalize(adjust(ospa.join(image_dir, image_path)))

        level += 1
        lookup_dir = ospa.join(start_dir, "../" * level)

    return '.'


def adjust_image_paths(content, lookup_dir, target_dir):
    """Locate all images referenced in a string and replace their paths
    with valid Markdown paths that point to the images relative to the target
    directory.

    ``lookup_dir`` is the starting point to look for the images—the directory
    with the document.
    """

    def sub(image):
        image_caption = image.group("caption")
        image_path = image.group("path")

        adjusted_image_path = find_image(image_path, lookup_dir, target_dir)

        return "![%s](%s)" % (image_caption, adjusted_image_path)

    return IMAGE_PATTERN.sub(sub, content)


def find_incl_file(incl_file_name, lookup_dir):
    lookup_dir_contents = os.listdir(lookup_dir)

    if incl_file_name in lookup_dir_contents:
        return ospa.join(lookup_dir, incl_file_name)
    else:
        child_dirs = (
            child for child in os.listdir(lookup_dir)
            if ospa.isdir(ospa.join(lookup_dir, child))
            and child not in IGNORE_DIRS
        )
        for child_dir in child_dirs:
            incl_file = find_incl_file(
                incl_file_name,
                ospa.join(lookup_dir, child_dir)
            )

            if incl_file:
                return incl_file

        return None


def process_local_include(path, from_heading, to_heading, options, sources_dir,
                          target_dir):
    """Replace a local include statement with the file content. Necessary
    adjustments are applied to the content: cut between certain headings,
    strip the top heading, set heading level.
    """

    incl_file_path = ospa.join(sources_dir, path)
    incl_file_dir, incl_file_name = ospa.split(incl_file_path)

    if incl_file_name.startswith('^'):
        adjusted_incl_file_path = find_incl_file(
            incl_file_name[1:],
            incl_file_dir
        )

        if not adjusted_incl_file_path:
            raise FileNotFoundError(
                errno.ENOENT, os.strerror(errno.ENOENT), incl_file_path
            )

        incl_file_dir, incl_file_name = ospa.split(adjusted_incl_file_path)

    else:
        adjusted_incl_file_path = incl_file_path

    with open(adjusted_incl_file_path, encoding="utf8") as incl_file:
        incl_content = incl_file.read()

        incl_content = adjust_headings(
            incl_content,
            from_heading,
            to_heading,
            options
        )

        incl_content = adjust_image_paths(
            incl_content,
            incl_file_dir,
            target_dir
        )

    return incl_content


def process_remote_include(repo, revision, path, from_heading, to_heading,
                           options, sources_dir, target_dir):
    """Please a remote include statement with the file content. This involves
    cloning or updating the git repository with the file and processing
    the include as a regular local one.
    """

    repo_path = gitutils.sync_repo(repo, target_dir, revision)

    return process_local_include(
        ospa.relpath(ospa.join(repo_path, path), sources_dir),
        from_heading,
        to_heading,
        options,
        sources_dir,
        target_dir
    )


def process_includes(content, sources_dir, target_dir, cfg):
    """Replace all include statements with the respective file content."""

    def sub(include):
        try:
            if include.group("repo"):
                repo = include.group("repo")
                repo_url = cfg.get("git", {}).get(repo) or repo

                return process_remote_include(
                    repo_url,
                    include.group("revision") or "master",
                    include.group("path"),
                    include.group("from_heading"),
                    include.group("to_heading"),
                    extract_options(include.group("options")),
                    sources_dir,
                    target_dir
                )
            else:
                return process_local_include(
                    include.group("path"),
                    include.group("from_heading"),
                    include.group("to_heading"),
                    extract_options(include.group("options")),
                    sources_dir,
                    target_dir
                )

        except FileNotFoundError as exception:
            print(
                Fore.YELLOW + "\nWarning: File '%s' not found." % exception.filename
            )
            return include.group(0).replace("{{", "<<")

    def restore_ignored_includes(ignored_include):
        return ignored_include.group(0).replace("<<", "{{")

    result = INCLUDE_PATTERN.sub(sub, content)

    if INCLUDE_PATTERN.search(result):
        return process_includes(result, sources_dir, target_dir, cfg)
    else:
        return IGNORED_INCLUDE_PATTERN.sub(restore_ignored_includes, result)



"""PlantUML processor."""

import subprocess
from os.path import join
from os import makedirs

from colorama import Fore


PLANTUML_COMMAND = "plantuml"
SUBDIR_NAME = "plantuml"


def process_diagram(caption, body, src_dir, diag_dir_name, diag_id):
    """Save diagram body to .diag file, draw PNG from it with plantuml,
    and return the image ref."""

    diag_src_path = join(src_dir, diag_dir_name, SUBDIR_NAME, "%s.diag" % diag_id)

    makedirs(join(src_dir, diag_dir_name, SUBDIR_NAME), exist_ok=True)

    with open(diag_src_path, 'w', encoding="utf8") as diag_src:
        diag_src.write(body)

    try:
        subprocess.check_output(
            "%s %s" % (PLANTUML_COMMAND, diag_src_path),
            stderr=subprocess.PIPE,
            shell=True
        )
    except subprocess.CalledProcessError as exception:
        print(
            Fore.YELLOW + "\nWarning: Processing of diagram %s failed: %s"
            % (diag_src_path, exception)
        )

    return "![%s](%s/%s/%s.png)" % (caption, diag_dir_name, SUBDIR_NAME, diag_id)



"""Wrapper around Pandoc. Used by builder."""

from __future__ import print_function

import sys
import subprocess

from colorama import Fore

from . import gitutils

PANDOC_PATH = "pandoc"

FROM_PARAMS = "-f markdown_strict+simple_tables+multiline_tables+grid_tables+\
pipe_tables+table_captions+fenced_code_blocks+line_blocks+definition_lists+\
all_symbols_escapable+strikeout+superscript+subscript+\
lists_without_preceding_blankline+implicit_figures+raw_tex+citations+\
tex_math_dollars+header_attributes+auto_identifiers+startnum+footnotes+\
inline_notes+fenced_code_attributes+intraword_underscores+escaped_line_breaks+\
backtick_code_blocks+startnum+fancy_lists"

LATEX_PARAMS = "--no-tex-ligatures --smart --normalize --listings --number-sections \
--latex-engine=xelatex"


def generate_variable(key, value):
    """Generate a ``--variable key=value`` entry."""

    return '--variable "%s"="%s"' % (key, value)

def generate_flag(key, value):
    """Generate a ``--variavle key`` entry."""

    if value == "true":
        return '--variable "%s"' % key
    else:
        return ''

def generate_command(params, output_file, src_file, cfg, set_template=False):
    """Generate the entire Pandoc command with params to invoke."""

    params = ["-o " + output_file, FROM_PARAMS, LATEX_PARAMS, params]

    for key, value in cfg.items():
        if key in ("title", "second_title", "year"):
            params.append(generate_variable(key, value))
        elif key in ("date", "title_page", "tof", "toc"):
            params.append(generate_flag(key, value))
        elif key == "template":
            if set_template:
                params.append('--template="%s.tex"' % value)
        elif key == "lang":
            if value in ("russian", "english"):
                params.append(generate_variable(value, "true"))
            else:
                params.append(generate_variable("russian", "true"))
        elif key == "version":
            if value == "auto":
                version = gitutils.get_version()
                if version:
                    params.append(generate_variable(key, version))
            else:
                params.append(generate_variable(key, value))
        elif key == "company":
            params.append(generate_variable(key, value))
        elif key == "filters":
            for filt in value:
                params.append("-F %s" % filt)
        elif key == "file_name":
            pass
        elif key in ("git", "chapters"):
            pass
        elif key in ("type", "alt_doc_type"):
            print(Fore.YELLOW + "Warning: Deprecated config key: %s" % key)
        else:
            print(Fore.YELLOW + "Warning: Unsupported config key: %s" % key)

    return ' '.join([PANDOC_PATH] + params + [src_file])


def run(command, src_dir):
    """Invoke the Pandoc executable with the generated params."""

    print("Baking output... ", end='')
    sys.stdout.flush()

    try:
        subprocess.check_output(
            command,
            cwd=src_dir,
            stderr=subprocess.PIPE,
            shell=True
        )

        print("Done!")

    except subprocess.CalledProcessError as e:
        quit(e.stderr.decode())


def to_pdf(src_file, output_file, tmp_path, cfg):
    """Convert Markdown to PDF via Pandoc."""

    pandoc_command = generate_command(
        "-t latex",
        output_file,
        src_file,
        cfg,
        set_template=True
    )
    run(pandoc_command, tmp_path)


def to_docx(src_file, output_file, tmp_path, cfg):
    """Convert Markdown to Docx via Pandoc."""

    pandoc_command = generate_command(
        '--reference-docx="ref.docx"',
        output_file,
        src_file,
        cfg
    )
    run(pandoc_command, tmp_path)


def to_odt(src_file, output_file, tmp_path, cfg):
    """Convert Markdown to OpenDocument via Pandoc."""

    pandoc_command = generate_command(
        '--reference-odt="ref.odt"',
        output_file,
        src_file,
        cfg
    )
    run(pandoc_command, tmp_path)


def to_tex(src_file, output_file, tmp_path, cfg):
    """Convert Markdown to TeX via Pandoc."""

    pandoc_command = generate_command(
        "-t latex",
        output_file,
        src_file,
        cfg,
        set_template=True
    )
    run(pandoc_command, tmp_path)



"""Foliant: Markdown to PDF, Docx, and LaTeX generator powered by Pandoc.

Usage:
  foliant (build | make) <target> [--path=<project-path>]
  foliant (upload | up) <document> [--secret=<client_secret*.json>]
  foliant (-h | --help)
  foliant --version

Options:
  -h --help                          Show this screen.
  -v --version                       Show version.
  -p --path=<project-path>           Path to your project [default: .].
  -s --secret=<client_secret*.json>  Path to Google app's client secret file.
"""

from docopt import docopt
import colorama
from colorama import Fore

from . import builder, uploader
from . import __version__ as foliant_version


def main():
    """Handles command-line params and runs the respective core function."""

    colorama.init(autoreset=True)

    args = docopt(__doc__, version="Foliant %s (Python)" % foliant_version)

    if args["build"] or args["make"]:
        result = builder.build(args["<target>"], args["--path"])

    elif args["upload"] or args["up"]:
        result = uploader.upload(args["<document>"])

    print("---")
    print(Fore.GREEN + "Result: %s" % result)

    colorama.deinit()



import os.path
import webbrowser
import pydrive.auth, pydrive.drive


def upload(document):
    """Upload .docx file to Google Drive and return a web view link to it."""

    auth = pydrive.auth.GoogleAuth()
    auth.CommandLineAuth()

    gdrive = pydrive.drive.GoogleDrive(auth)

    gdoc = gdrive.CreateFile({
        "title": os.path.splitext(os.path.basename(document))[0]
    })
    gdoc.SetContentFile(document)
    gdoc.Upload({"convert": True})

    webbrowser.open(gdoc["alternateLink"])

    return gdoc["alternateLink"]



"""Document uploader for foliant. Implements "upload" subcommand."""

from . import gdrive

def upload(document_path):
    """Upload .docx file to Google Drive."""

    return gdrive.upload(document_path)



"""Document builder for foliant. Implements "build" subcommand."""

import sys
import os, shutil, json
from os.path import join, splitext
from datetime import date

from colorama import Fore
import yaml

from . import gitutils, pandoc, uploader, diagrams, includes


SOURCES_DIR_NAME = "sources"
IMAGES_DIR_NAME = "images"
TEMPLATES_DIR_NAME = "templates"
REFERENCES_DIR_NAME = "references"
TMP_DIR_NAME = "foliantcache"
MERGED_SRC_FILE_NAME = "output.md"
CONFIG_FILE_NAME = "config.json"


def copy_dir_content(src, dest):
    """Recusrively copy directory content to another directory."""

    if not os.path.exists(src):
        return

    for child in os.listdir(src):
        if os.path.isfile(join(src, child)):
            shutil.copy(join(src, child), dest)
        elif os.path.isdir(join(src, child)):
            if os.path.exists(join(dest, child)):
                shutil.rmtree(join(dest, child))
            shutil.copytree(join(src, child), join(dest, child))


def get_version(cfg):
    """Extract version from config or generate it from git tag and revcount.
    Append current date.
    """

    components = []

    git_version = gitutils.get_version() if cfg["version"] == "auto" else cfg["version"]
    if git_version:
        components.append(git_version)

    if cfg["date"] == "true":
        components.append(date.today().strftime("%d-%m-%Y"))

    return '-'.join(components)


def get_title(cfg):
    """Generate file name from config: slugify the title and add version."""

    components = []

    file_name = cfg.get("file_name", cfg["title"].replace(' ', '_'))
    components.append(file_name)

    version = get_version(cfg)
    if version:
        components.append(version)

    return '_'.join(components)


def collect_source(project_dir, target_dir, src_file, cfg):
    """Copy .md files, images, templates, and references from the project
    directory to a temporary directory.
    """

    print("Collecting source... ", end='')
    sys.stdout.flush()

    with open(join(target_dir, src_file), 'w+', encoding="utf8") as src:
        chapters = cfg.get("chapters")

        if not chapters:
            try:
                with open(join(project_dir, "main.yaml"), encoding="utf8") as chapters_file:
                    chapters = yaml.load(chapters_file)["chapters"]

                print(
                    Fore.YELLOW
                    + "\nWarning: main.yaml is deprecated."
                    + " Use `chapters` key in config.json instead."
                )

            except FileNotFoundError:
                print(
                    Fore.RED + "\nCritical error: `chapters` key is missing in config.json."
                )
                print(Fore.RESET)
                sys.exit()

        for chapter_name in chapters:
            chapter_file = chapter_name + ".md"
            with open(
                join(project_dir, SOURCES_DIR_NAME, chapter_file),
                encoding="utf8"
            ) as chapter:
                src.write(
                    includes.process_includes(
                        chapter.read(),
                        join(project_dir, SOURCES_DIR_NAME),
                        target_dir,
                        cfg
                    ) + "\n"
                )

    copy_dir_content(join(project_dir, SOURCES_DIR_NAME, IMAGES_DIR_NAME), target_dir)
    copy_dir_content(join(project_dir, TEMPLATES_DIR_NAME), target_dir)
    copy_dir_content(join(project_dir, REFERENCES_DIR_NAME), target_dir)

    print("Done!")


def build(target_format, project_dir):
    """Convert source Markdown to the target format using Pandoc."""

    os.makedirs(TMP_DIR_NAME, exist_ok=True)

    with open(join(project_dir, CONFIG_FILE_NAME), encoding="utf8") as cfg_file:
        cfg = json.load(cfg_file)

    output_title = get_title(cfg)

    collect_source(project_dir, TMP_DIR_NAME, MERGED_SRC_FILE_NAME, cfg)
    diagrams.process_diagrams(TMP_DIR_NAME, MERGED_SRC_FILE_NAME)

    if target_format.startswith('p'):
        output_file = output_title + ".pdf"
        pandoc.to_pdf(MERGED_SRC_FILE_NAME, output_file, TMP_DIR_NAME, cfg)
        shutil.copy(join(TMP_DIR_NAME, output_file), output_file)
    elif target_format.startswith('d'):
        output_file = output_title + ".docx"
        pandoc.to_docx(MERGED_SRC_FILE_NAME, output_file, TMP_DIR_NAME, cfg)
        shutil.copy(join(TMP_DIR_NAME, output_file), output_file)
    elif target_format.startswith('o'):
        output_file = output_title + ".odt"
        pandoc.to_odt(MERGED_SRC_FILE_NAME, output_file, TMP_DIR_NAME, cfg)
        shutil.copy(join(TMP_DIR_NAME, output_file), output_file)
    elif target_format.startswith('t'):
        output_file = output_title + ".tex"
        pandoc.to_tex(MERGED_SRC_FILE_NAME, output_file, TMP_DIR_NAME, cfg)
        shutil.copy(join(TMP_DIR_NAME, output_file), output_file)
    elif target_format.startswith('m'):
        output_file = output_title + ".md"
        shutil.copy(join(TMP_DIR_NAME, MERGED_SRC_FILE_NAME), output_file)
    elif target_format.startswith('g'):
        output_file = output_title + ".docx"
        pandoc.to_docx(MERGED_SRC_FILE_NAME, output_file, TMP_DIR_NAME, cfg)
        output_file = uploader.upload(join(TMP_DIR_NAME, output_file))
    else:
        raise RuntimeError("Invalid target: %s" % target_format)

    return output_file



from setuptools import setup, find_packages

import foliant


def readme():
    try:
        with open("README.rst", encoding="utf8") as f:
            return f.read()
    except IOError:
        pass

setup(
    name="foliant",
    version=foliant.__version__,
    url="https://github.com/foliant-docs/foliant",
    download_url="https://pypi.org/project/foliant",
    license="MIT",
    author="Konstantin Molchanov",
    author_email="moigagoo@live.com",
    description="Markdown-based, Pandoc-powered documentation generator.",
    long_description=readme(),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Documentation",
        "Topic :: Utilities",
    ],
    packages=find_packages(),
    platforms="any",
    install_requires=[
        "PyDrive>=1.2.1",
        "docopt",
        "seqdiag",
        "colorama",
        "PyYAML"
    ],
    entry_points={
        "console_scripts": [
            "foliant=foliant.cli:main"
        ]
    }
)



"""Diagram processor."""

import sys
import os
from os.path import join
import re
from uuid import uuid4

from . import seqdiag, plantuml


DIAG_BACKENDS = {
    "seqdiag": seqdiag.process_diagram,
    "plantuml": plantuml.process_diagram,
}
DIAG_PATTERN = re.compile(
    r"^```(?P<kind>%s)(?P<caption>.*?)$(?P<body>.*?)```$"
    % "|".join(DIAG_BACKENDS.keys()),
    flags=re.MULTILINE|re.DOTALL
)
DIAG_DIR_NAME = "diagrams"


def process_diagrams(src_dir, src_file):
    """Find diagram definitions and replace them with image refs.
    The definitions are fed to processors that convert them into images"""

    def sub(include):
        return DIAG_BACKENDS[include.group("kind")](
            include.group("caption").lstrip(),
            include.group("body"),
            src_dir,
            DIAG_DIR_NAME,
            uuid4()
        )

    print("Drawing diagrams... ", end='')
    sys.stdout.flush()

    os.makedirs(join(src_dir, DIAG_DIR_NAME), exist_ok=True)

    src_path = join(src_dir, src_file)

    with open(src_path, encoding="utf8") as src:
        result = DIAG_PATTERN.sub(sub, src.read())

    with open(src_path, 'w', encoding="utf8") as src:
        src.write(result)

    print("Done!")



""""Wrapper around a few git commands. Used by builder to determin document
version.
"""

import sys
import subprocess
from os.path import join


def get_version():
    """Generate document version based on git tag and number of revisions."""

    components = []

    try:
        components.append(
            subprocess.check_output(
                "git describe --abbrev=0",
                stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                shell=True
            ).strip().decode()
        )
        components.append(
            subprocess.check_output(
                "git rev-list --count master",
                stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                shell=True
            ).strip().decode()
        )
    except:
        pass

    return '.'.join(components) if components else None


def sync_repo(repo_url, target_dir, revision="master"):
    """Clone git repository if it's not cloned yet or pull changes otherwise.
    """

    repo_name = repo_url.split('/')[-1].split('.')[0]
    repo_path = join(target_dir, repo_name)

    if subprocess.run(
        "git clone %s %s" % (repo_url, repo_path),
        stdout=subprocess.PIPE, stderr=subprocess.PIPE,
        shell=True
    ).returncode != 0:
        subprocess.run(
            "git pull",
            cwd=repo_path,
            stdout=subprocess.PIPE, stderr=subprocess.PIPE,
            shell=True
        )

    subprocess.run(
        "git checkout %s" % revision,
        cwd=repo_path,
        stdout=subprocess.PIPE, stderr=subprocess.PIPE,
        shell=True
    )

    return repo_path



"""
**Foliant** is a documentation generator that builds PDF, Docx, and TeX output
from a single Markdown source. It also uploads Docx files to Google Drive
as Google Documents.
"""

__version__ = "0.4.9"
